@@ -261,7 +261,7 @@ p_netk_get_text_property (Window xwindow, Atom atom)
         {
             xfce_utf8_remove_controls((gchar *) retval, -1, NULL);
         }
-        if (text.nitems > 0)
+        if (text.value)
             XFree (text.value);
     }
     else
@@ -771,6 +771,7 @@ p_netk_close (Screen * screen, Window xwindow)
     xev.xclient.data.l[0] = 0;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -825,6 +826,7 @@ p_netk_keyboard_size (Screen * screen, Window xwindow)
     xev.xclient.data.l[1] = 0;  /* unused */
     xev.xclient.data.l[2] = _NET_WM_MOVERESIZE_SIZE_KEYBOARD;
     xev.xclient.data.l[3] = 0;  /* unused */
+    
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -850,6 +852,7 @@ p_netk_change_state (Screen * screen, Window xwindow, gboolean add,
     xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
     xev.xclient.data.l[1] = state1;
     xev.xclient.data.l[2] = state2;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -870,6 +873,7 @@ p_netk_change_workspace (Screen * screen, Window xwindow, int new_space)
     xev.xclient.data.l[0] = new_space;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -890,6 +894,7 @@ p_netk_activate (Screen * screen, Window xwindow)
     xev.xclient.data.l[0] = 0;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -910,6 +915,7 @@ p_netk_activate_workspace (Screen * screen, int new_active_space)
     xev.xclient.data.l[0] = new_active_space;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -930,6 +936,7 @@ p_netk_change_viewport (Screen * screen, int x, int y)
     xev.xclient.data.l[0] = x;
     xev.xclient.data.l[1] = y;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display,
                 RootWindowOfScreen (screen),
@@ -952,6 +959,7 @@ p_netk_toggle_showing_desktop (Screen * screen, gboolean show)
     xev.xclient.data.l[0] = show != FALSE;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (DisplayOfScreen (screen),
                 RootWindowOfScreen (screen),
@@ -1008,23 +1016,42 @@ p_netk_get_pid (Window xwindow)
         return val;
 }
 
+static gchar *
+netk_utf8_strndup (const gchar *src, gssize max_len)
+{
+    const gchar *s = src;
+
+    if (max_len <= 0)
+	return g_strdup (src);
+
+    while (max_len && *s)
+    {
+	s = g_utf8_next_char (s);
+	max_len--;
+    }
+
+    return g_strndup (src, s - src);
+}
+
 char *
 p_netk_get_name (Window xwindow)
 {
-    char *name;
+    char *s, *name = NULL;
 
-    name =
-        p_netk_get_utf8_property (xwindow,
-                                  p_netk_atom_get ("_NET_WM_VISIBLE_NAME"));
+    s = p_netk_get_utf8_property (xwindow, p_netk_atom_get ("_NET_WM_VISIBLE_NAME"));
 
-    if (name == NULL)
-        name =
-            p_netk_get_utf8_property (xwindow,
-                                      p_netk_atom_get ("_NET_WM_NAME"));
+    if (s == NULL)
+        s = p_netk_get_utf8_property (xwindow, p_netk_atom_get ("_NET_WM_NAME"));
 
-    if (name == NULL)
-        name = p_netk_get_text_property (xwindow, XA_WM_NAME);
+    if (s == NULL)
+        s = p_netk_get_text_property (xwindow, XA_WM_NAME);
 
+    if (s != NULL)
+    {
+        name = netk_utf8_strndup (s, NETK_MAX_TITLE_LENGTH);
+        g_free (s);
+    }
+    
     return name;
 }
 
@@ -1654,7 +1681,7 @@ p_netk_icon_cache_new (void)
 {
     NetkIconCache *icon_cache;
 
-    icon_cache = g_new (NetkIconCache, 1);
+    icon_cache = g_new0 (NetkIconCache, 1);
 
     icon_cache->origin = USING_NO_ICON;
     icon_cache->prev_pixmap = None;
