@@ -1,5 +1,5 @@
-/* $Id: xfce-exec.c 1683 2005-03-06 08:33:39Z jasper $
- * 
+/* $Id: xfce-exec.c 19324 2006-01-06 23:01:09Z dannym $
+ *  
  * Copyright (C) 2004 Jasper Huijsmans <jasper@xfce.org>
  * startup notification added by Olivier Fourdan based on gnome-desktop
  * developed by Elliot Lee <sopwith@redhat.com> and Sid Vicious
@@ -32,6 +32,9 @@
 #include <stddef.h>
 #endif
 
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+
 #include <gdk/gdk.h>
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
@@ -41,6 +44,8 @@
 #define STARTUP_TIMEOUT (30 /* seconds */ * 1000)
 #endif
 
+#include <libxfce4util/libxfce4util.h>
+
 #include "xfce-exec.h"
 
 extern char **environ;
@@ -208,8 +213,214 @@ free_startup_timeout (void)
     atexit_registered = FALSE;
 }
 
+static gint
+get_active_workspace_number ()
+{
+    GdkDisplay *gdpy = gdk_display_get_default();
+    Display *dpy = GDK_DISPLAY_XDISPLAY(gdpy);
+    GdkScreen *gscreen = gdk_display_get_default_screen(gdpy);
+    GdkWindow *groot = gdk_screen_get_root_window(gscreen);
+    Atom _NET_CURRENT_DESKTOP = XInternAtom(dpy, "_NET_CURRENT_DESKTOP", False);
+    Atom _WIN_WORKSPACE = XInternAtom(dpy, "_WIN_WORKSPACE", False);
+    Atom type_ret = None;
+    int format_ret = 0;
+    unsigned long nitems_ret = 0;
+    unsigned long bytes_after_ret = 0;
+    unsigned int *prop_ret = NULL;
+    gint ws_num = 0;
+    
+    gdk_error_trap_push();
+    
+    if(XGetWindowProperty(dpy, GDK_WINDOW_XWINDOW(groot), _NET_CURRENT_DESKTOP,
+            0, 32, False, XA_CARDINAL, &type_ret, &format_ret, &nitems_ret,
+            &bytes_after_ret, (unsigned char **)&prop_ret) != Success)
+    {
+        if(XGetWindowProperty(dpy, GDK_WINDOW_XWINDOW(groot), _WIN_WORKSPACE,
+            0, 32, False, XA_CARDINAL, &type_ret, &format_ret, &nitems_ret,
+            &bytes_after_ret, (unsigned char **)&prop_ret) != Success)
+        {
+            return 0;
+        }
+    }
+    
+    gdk_error_trap_pop();
+    
+    if(type_ret == None || format_ret == 0 || !prop_ret) {
+        if(prop_ret)
+            XFree(prop_ret);
+        return 0;
+    }
+    
+    ws_num = *prop_ret;
+    XFree(prop_ret);
+    
+    return ws_num;
+}
+
 #endif /* HAVE_LIBSTARTUP_NOTIFICATION */
 
+static gboolean
+real_xfce_exec_with_envp (const char *cmd, 
+		     gboolean in_terminal, 
+		     gboolean use_sn,
+		     GError ** error,
+		     char **envp,
+                     gboolean async)
+{
+    gchar **free_envp = NULL;
+    gchar **argv = NULL;
+    gboolean success;
+    char *realcmd;
+    gint exit_status;
+    
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+    SnLauncherContext *sn_context = NULL;
+    SnDisplay *sn_display = NULL;
+
+    if (!atexit_registered)
+    {
+	g_atexit (free_startup_timeout);
+	atexit_registered = TRUE;
+    }
+#endif
+
+    if (!cmd)
+        return FALSE;
+    
+    realcmd = g_strdup (cmd);
+    
+    if (realcmd[0] == '"')
+    {
+        int i;
+
+        for (i = 1; realcmd[i-1] != '\0'; ++i)
+        {
+            if (realcmd[i] != '"')
+                realcmd[i-1] = realcmd[i-1];
+            else {                
+                realcmd[i-1] = realcmd[i] = ' ';
+                break;
+            }
+        }
+    }
+    
+    if (g_path_is_absolute (realcmd) && 
+            g_file_test (realcmd, G_FILE_TEST_IS_DIR))
+    {
+        argv = g_new (char *, 3);
+
+        if (in_terminal)
+            argv[0] = g_strdup ("xfterm4");
+	else
+	    argv[0] = g_strdup ("xftree4");
+        
+        argv[1] = realcmd;
+        argv[2] = NULL;
+    }
+    else if (in_terminal)
+    {
+        argv = g_new (char *, 4);
+        
+        argv[0] = g_strdup ("xfterm4");
+        argv[1] = g_strdup ("-e");
+	argv[2] = realcmd;
+        argv[3] = NULL;
+    }
+    else
+    {
+        if (!g_shell_parse_argv (realcmd, NULL, &argv, error))
+        {
+            g_free (realcmd);
+
+            return FALSE;
+        }
+        
+        g_free (realcmd);
+    }    
+
+    free_envp = NULL;
+
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+    if (use_sn)
+    {
+	sn_display = sn_display_new (gdk_display, sn_error_trap_push,
+				     sn_error_trap_pop);
+
+	if (sn_display != NULL)
+	{
+	    sn_context = 
+		sn_launcher_context_new (sn_display,
+					 DefaultScreen (gdk_display));
+	    if ((sn_context != NULL)
+		&& !sn_launcher_context_get_initiated (sn_context))
+	    {
+		gint workspace = get_active_workspace_number ();
+		DBG("got workspace: %d", workspace);
+		sn_launcher_context_set_workspace (sn_context, workspace);
+		sn_launcher_context_set_binary_name (sn_context, argv[0]);
+		sn_launcher_context_initiate (sn_context,
+					      g_get_prgname () ?
+					      g_get_prgname () : "unknown",
+					      argv[0], 
+					      gtk_get_current_event_time ());
+		free_envp =
+		    make_spawn_environment_for_sn_context (sn_context, envp);
+	    }
+	}
+    }
+#endif
+
+    if (async)
+    {
+        success = g_spawn_async_with_pipes (NULL, argv, 
+                                            free_envp ? free_envp : envp,
+                                            G_SPAWN_SEARCH_PATH, 
+                                            NULL, NULL, NULL,
+                                            NULL, NULL, NULL,
+                                            error);
+    }
+    else
+    {
+        success = g_spawn_sync (NULL, argv, 
+                                free_envp ? free_envp : envp,
+                                G_SPAWN_SEARCH_PATH, 
+                                NULL, NULL,
+                                NULL, NULL,
+                                &exit_status, error);
+    }
+    
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+    if (use_sn)
+    {
+	if (sn_context != NULL)
+	{
+	    if (!success)
+	    {
+		sn_launcher_context_complete (sn_context); /* end sequence */
+	    }
+	    else
+	    {
+		add_startup_timeout (sn_context);
+		sn_launcher_context_unref (sn_context);
+	    }
+	}
+	if (free_envp)
+	{
+	    g_strfreev (free_envp);
+	    free_envp = NULL;
+	}
+	if (sn_display)
+	{
+	    sn_display_unref (sn_display);
+	}
+    }
+#endif /* HAVE_LIBSTARTUP_NOTIFICATION */
+
+    g_strfreev (argv);
+
+    return success;
+}
+
 /**
  * xfce_exec
  * @cmd         : command line to run
@@ -226,7 +437,8 @@ gboolean xfce_exec (const char *cmd,
 		    gboolean use_sn, 
 		    GError **error)
 {
-    return xfce_exec_with_envp (cmd, in_terminal, use_sn, error, environ);
+    return real_xfce_exec_with_envp (cmd, in_terminal, use_sn, error,
+                                     environ, TRUE);
 }
 
 /**
@@ -248,140 +460,8 @@ xfce_exec_with_envp (const char *cmd,
 		     GError ** error,
 		     char **envp)
 {
-    gchar *execute = NULL;
-    gchar **free_envp = NULL;
-    gchar **argv = NULL;
-    gboolean success;
-    char *realcmd;
-    
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    SnLauncherContext *sn_context = NULL;
-    SnDisplay *sn_display = NULL;
-
-    if (!atexit_registered)
-    {
-	g_atexit (free_startup_timeout);
-	atexit_registered = TRUE;
-    }
-#endif
-
-    if (!cmd)
-        return FALSE;
-    
-    realcmd = g_strdup (cmd);
-    
-    if (realcmd[0] == '"')
-    {
-        int i;
-
-        for (i = 1; realcmd[i-1] != '\0'; ++i)
-        {
-            if (realcmd[i] != '"')
-                realcmd[i-1] = realcmd[i-1];
-            else {                
-                realcmd[i-1] = realcmd[i] = ' ';
-                break;
-            }
-        }
-    }
-    
-    if (g_path_is_absolute (realcmd) && 
-            g_file_test (realcmd, G_FILE_TEST_IS_DIR))
-    {
-	if (in_terminal)
-	{
-	    execute = g_strconcat ("xfterm4 ", realcmd, NULL);
-	}
-	else
-	{
-	    execute = g_strconcat ("xftree4 ", realcmd, NULL);
-	}
-    }
-    else if (in_terminal)
-    {
-	execute = g_strconcat ("xfterm4 -e ", realcmd, NULL);
-    }
-    else
-    {
-	execute = g_strdup (realcmd);
-    }
-
-    g_free (realcmd);
-
-    if (!g_shell_parse_argv (execute, NULL, &argv, error))
-    {
-	g_free (execute);
-
-	return FALSE;
-    }
-
-    free_envp = NULL;
-
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    if (use_sn)
-    {
-	sn_display = sn_display_new (gdk_display, sn_error_trap_push,
-				     sn_error_trap_pop);
-
-	if (sn_display != NULL)
-	{
-	    sn_context = 
-		sn_launcher_context_new (sn_display,
-					 DefaultScreen (gdk_display));
-	    if ((sn_context != NULL)
-		&& !sn_launcher_context_get_initiated (sn_context))
-	    {
-		sn_launcher_context_set_binary_name (sn_context, execute);
-		sn_launcher_context_initiate (sn_context,
-					      g_get_prgname () ?
-					      g_get_prgname () : "unknown",
-					      argv[0], CurrentTime);
-		free_envp =
-		    make_spawn_environment_for_sn_context (sn_context, envp);
-	    }
-	}
-    }
-#endif
-
-    g_free (execute);
-
-    success = g_spawn_async_with_pipes (NULL, 
-	    				argv, free_envp ? free_envp : envp,
-					G_SPAWN_SEARCH_PATH, 
-					NULL, NULL, NULL,
-					NULL, NULL, NULL,
-					error);
-
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    if (use_sn && success)
-    {
-	if (sn_context != NULL)
-	{
-	    if (!success)
-	    {
-		sn_launcher_context_complete (sn_context); /* end sequence */
-	    }
-	    else
-	    {
-		add_startup_timeout (sn_context);
-		sn_launcher_context_unref (sn_context);
-	    }
-	}
-	if (free_envp)
-	{
-	    g_strfreev (free_envp);
-	    free_envp = NULL;
-	}
-	if (sn_display)
-	{
-	    sn_display_unref (sn_display);
-	}
-    }
-#endif /* HAVE_LIBSTARTUP_NOTIFICATION */
-
-    g_strfreev (argv);
-
-    return success;
+    return real_xfce_exec_with_envp (cmd, in_terminal, use_sn, error, envp,
+                                     TRUE);
 }
 
 /**
@@ -400,7 +480,8 @@ gboolean xfce_exec_sync (const char *cmd,
 			 gboolean use_sn, 
 			 GError **error)
 {
-    return xfce_exec_sync_with_envp (cmd, in_terminal, use_sn, error, environ);
+    return real_xfce_exec_with_envp (cmd, in_terminal, use_sn, error,
+                                     environ, FALSE);
 }
 
 /**
@@ -422,139 +503,6 @@ xfce_exec_sync_with_envp (const char *cmd,
 			  GError ** error,
 			  char **envp)
 {
-    gchar *execute = NULL;
-    gchar **free_envp = NULL;
-    gchar **argv = NULL;
-    gint exit_status;
-    gboolean success;
-    char *realcmd;
-    
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    SnLauncherContext *sn_context = NULL;
-    SnDisplay *sn_display = NULL;
-
-    if (!atexit_registered)
-    {
-	g_atexit (free_startup_timeout);
-	atexit_registered = TRUE;
-    }
-#endif
-
-    if (!cmd)
-        return FALSE;
-    
-    realcmd = g_strdup (cmd);
-    
-    if (realcmd[0] == '"')
-    {
-        int i;
-
-        for (i = 1; realcmd[i-1] != '\0'; ++i)
-        {
-            if (realcmd[i] != '"')
-                realcmd[i-1] = realcmd[i-1];
-            else {                
-                realcmd[i-1] = realcmd[i] = ' ';
-                break;
-            }
-        }
-    }
-    
-    if (g_path_is_absolute (realcmd) && 
-            g_file_test (realcmd, G_FILE_TEST_IS_DIR))
-    {
-	if (in_terminal)
-	{
-	    execute = g_strconcat ("xfterm4 ", realcmd, NULL);
-	}
-	else
-	{
-	    execute = g_strconcat ("xftree4 ", realcmd, NULL);
-	}
-    }
-    else if (in_terminal)
-    {
-	execute = g_strconcat ("xfterm4 -e ", realcmd, NULL);
-    }
-    else
-    {
-	execute = g_strdup (realcmd);
-    }
-
-    g_free (realcmd);
-
-    if (!g_shell_parse_argv (execute, NULL, &argv, error))
-    {
-	g_free (execute);
-
-	return FALSE;
-    }
-
-    free_envp = NULL;
-
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    if (use_sn)
-    {
-	sn_display = sn_display_new (gdk_display, sn_error_trap_push,
-				     sn_error_trap_pop);
-
-	if (sn_display != NULL)
-	{
-	    sn_context = 
-		sn_launcher_context_new (sn_display,
-					 DefaultScreen (gdk_display));
-	    if ((sn_context != NULL)
-		&& !sn_launcher_context_get_initiated (sn_context))
-	    {
-		sn_launcher_context_set_binary_name (sn_context, execute);
-		sn_launcher_context_initiate (sn_context,
-					      g_get_prgname () ?
-					      g_get_prgname () : "unknown",
-					      argv[0], CurrentTime);
-		free_envp =
-		    make_spawn_environment_for_sn_context (sn_context, envp);
-	    }
-	}
-    }
-#endif
-
-    g_free (execute);
-
-    success = g_spawn_sync (NULL, 
-			    argv, free_envp ? free_envp : envp,
-			    G_SPAWN_SEARCH_PATH, 
-			    NULL, NULL,
-			    NULL, NULL,
-			    &exit_status, error);
-
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    if (use_sn && success)
-    {
-	if (sn_context != NULL)
-	{
-	    if (!success)
-	    {
-		sn_launcher_context_complete (sn_context); /* end sequence */
-	    }
-	    else
-	    {
-		add_startup_timeout (sn_context);
-		sn_launcher_context_unref (sn_context);
-	    }
-	}
-	if (free_envp)
-	{
-	    g_strfreev (free_envp);
-	    free_envp = NULL;
-	}
-	if (sn_display)
-	{
-	    sn_display_unref (sn_display);
-	}
-    }
-#endif /* HAVE_LIBSTARTUP_NOTIFICATION */
-
-    g_strfreev (argv);
-
-    return success;
+    return real_xfce_exec_with_envp (cmd, in_terminal, use_sn, error, envp,
+                                     FALSE);
 }
